// eslint-disable-next-line no-unused-vars
import { EVENT_CROP, EVENT_CROP_END, EVENT_CROP_MOVE, EVENT_CROP_START, EVENT_DBLCLICK, EVENT_POINTER_DOWN, EVENT_POINTER_MOVE, EVENT_POINTER_UP, EVENT_RESIZE, EVENT_WHEEL, EVENT_ZOOM } from '../constants'
// eslint-disable-next-line no-unused-vars
import { addListener, isFunction, removeListener } from '../utilities'

export default {
  methods: {
    bind() {
      // eslint-disable-next-line no-unused-vars
      const { cropper, element } = this

      if (isFunction(this.cropstart)) {
        addListener(element, EVENT_CROP_START, this.cropstart)
      }

      if (isFunction(this.cropmove)) {
        addListener(element, EVENT_CROP_MOVE, this.cropmove)
      }

      if (isFunction(this.cropend)) {
        addListener(element, EVENT_CROP_END, this.cropend)
      }

      if (isFunction(this.croping)) {
        addListener(element, EVENT_CROP, this.croping)
      }

      if (isFunction(this.zooming)) {
        addListener(element, EVENT_ZOOM, this.zooming)
      }

      addListener(cropper, EVENT_POINTER_DOWN, (this.onCropStart = this.cropStart.bind(this)))

      if (this.zoomable && this.zoomOnWheel) {
        addListener(cropper, EVENT_WHEEL, (this.onWheel = this.wheel.bind(this)), {
          passive: false,
          capture: true
        })
      }

      if (this.toggleDragModeOnDblclick) {
        addListener(cropper, EVENT_DBLCLICK, (this.onDblclick = this.dblclick.bind(this)))
      }

      addListener(
        element.ownerDocument,
        EVENT_POINTER_MOVE,
        (this.onCropMove = this.cropMove.bind(this))
      )
      addListener(
        element.ownerDocument,
        EVENT_POINTER_UP,
        (this.onCropEnd = this.cropEnd.bind(this))
      )

      if (this.responsive) {
        addListener(window, EVENT_RESIZE, (this.onResize = this.resize.bind(this)))
      }
    },
    unbindEvent() {
      const { element, cropper } = this

      if (isFunction(this.cropstart)) {
        removeListener(element, EVENT_CROP_START, this.cropstart)
      }

      if (isFunction(this.cropmove)) {
        removeListener(element, EVENT_CROP_MOVE, this.cropmove)
      }

      if (isFunction(this.cropend)) {
        removeListener(element, EVENT_CROP_END, this.cropend)
      }

      if (isFunction(this.croping)) {
        removeListener(element, EVENT_CROP, this.croping)
      }

      if (isFunction(this.zooming)) {
        removeListener(element, EVENT_ZOOM, this.zooming)
      }

      removeListener(cropper, EVENT_POINTER_DOWN, this.onCropStart)

      if (this.zoomable && this.zoomOnWheel) {
        removeListener(cropper, EVENT_WHEEL, this.onWheel, {
          passive: false,
          capture: true
        })
      }

      if (this.toggleDragModeOnDblclick) {
        removeListener(cropper, EVENT_DBLCLICK, this.onDblclick)
      }

      removeListener(element.ownerDocument, EVENT_POINTER_MOVE, this.onCropMove)
      removeListener(element.ownerDocument, EVENT_POINTER_UP, this.onCropEnd)

      if (this.responsive) {
        removeListener(window, EVENT_RESIZE, this.onResize)
      }
    }
  }
}
